@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import ast
 import hashlib
 import importlib
 import importlib.util
@@ -23,6 +24,7 @@ from typing import (
     List,
     NamedTuple,
     Optional,
+    Set,
     Tuple,
     TypeVar,
     Union,
@@ -31,8 +33,12 @@ from typing import (
 
 from robotcode.core.lsp.types import Position, Range
 from robotcode.language_server.robotframework.utils.ast_utils import (
+    HasError,
+    HasErrors,
     Token,
+    get_variable_token,
     range_from_token,
+    strip_variable_token,
 )
 from robotcode.language_server.robotframework.utils.markdownformatter import (
     MarkDownFormatter,
@@ -43,6 +49,7 @@ from robotcode.language_server.robotframework.utils.version import get_robot_ver
 from .entities import (
     ArgumentDefinition,
     ImportedVariableDefinition,
+    LibraryArgumentDefinition,
     NativeValue,
     SourceEntity,
     single_call,
@@ -216,6 +223,87 @@ RUN_KEYWORDS_MATCHER = KeywordMatcher(RUN_KEYWORDS_NAME)
 ALL_RUN_KEYWORDS_MATCHERS = [KeywordMatcher(e) for e in ALL_RUN_KEYWORDS]
 
 
+class TypeDocType(Enum):
+    ENUM = "Enum"
+    TYPED_DICT = "TypedDict"
+    CUSTOM = "Custom"
+    STANDARD = "Standard"
+
+
+@dataclass
+class TypedDictItem:
+    key: str
+    type: str
+    required: Optional[bool] = None
+
+
+@dataclass
+class EnumMember:
+    name: str
+    value: str
+
+
+@dataclass
+class TypeDoc:
+    type: str
+    name: str
+    doc: Optional[str] = None
+    accepts: List[str] = field(default_factory=list)
+    usages: List[str] = field(default_factory=list)
+    members: Optional[List[EnumMember]] = None
+    items: Optional[List[TypedDictItem]] = None
+
+    libname: Optional[str] = None
+    libtype: Optional[str] = None
+
+    doc_format: str = ROBOT_DOC_FORMAT
+
+    @single_call
+    def __hash__(self) -> int:
+        return hash(
+            (
+                self.type,
+                self.name,
+                self.libname,
+                self.libtype,
+                tuple(self.accepts),
+                tuple(self.usages),
+            )
+        )
+
+    def to_markdown(self, header_level: int = 2) -> str:
+        result = ""
+
+        result += f"##{'#'*header_level} {self.name} ({self.type})\n\n"
+
+        if self.doc:
+            result += f"###{'#'*header_level} Documentation:\n\n"
+            result += "\n\n"
+
+            if self.doc_format == ROBOT_DOC_FORMAT:
+                result += MarkDownFormatter().format(self.doc)
+            elif self.doc_format == REST_DOC_FORMAT:
+                result += convert_from_rest(self.doc)
+            else:
+                result += self.doc
+
+        if self.members:
+            result += f"\n\n###{'#'*header_level} Allowed Values:\n\n"
+            result += "- " + "\n- ".join(f"`{m.name}`" for m in self.members)
+
+        if self.items:
+            result += f"\n\n###{'#'*header_level} Dictionary Structure:\n\n"
+            result += "```\n{"
+            result += "\n ".join(f"'{m.key}': <{m.type}> {'# optional' if not m.required else ''}" for m in self.items)
+            result += "\n}\n```"
+
+        if self.accepts:
+            result += f"\n\n###{'#'*header_level} Converted Types:\n\n"
+            result += "- " + "\n- ".join(self.accepts)
+
+        return result
+
+
 @dataclass
 class Error:
     message: str
@@ -258,21 +346,21 @@ def robot_arg_repr(arg: Any) -> Optional[str]:
 
 
 @dataclass
-class KeywordArgumentDoc:
+class ArgumentInfo:
     name: str
     str_repr: str
     kind: KeywordArgumentKind
     required: bool
     default_value: Optional[Any] = None
-    types: Optional[Any] = None
+    types: Optional[List[str]] = None
 
     @staticmethod
-    def from_robot(arg: Any) -> KeywordArgumentDoc:
+    def from_robot(arg: Any) -> ArgumentInfo:
         from robot.running.arguments.argumentspec import ArgInfo
 
         robot_arg = cast(ArgInfo, arg)
 
-        return KeywordArgumentDoc(
+        return ArgumentInfo(
             name=robot_arg.name,
             default_value=robot_arg_repr(robot_arg),
             str_repr=str(arg),
@@ -283,11 +371,18 @@ class KeywordArgumentDoc:
 
     def __str__(self) -> str:
         prefix = ""
-        if self.kind == KeywordArgumentKind.VAR_POSITIONAL:
+        if self.kind == KeywordArgumentKind.POSITIONAL_ONLY_MARKER:
             prefix = "*"
+        elif self.kind == KeywordArgumentKind.NAMED_ONLY_MARKER:
+            prefix = "/"
         elif self.kind == KeywordArgumentKind.VAR_NAMED:
             prefix = "**"
-
+        elif self.kind == KeywordArgumentKind.VAR_POSITIONAL:
+            prefix = "*"
+        elif self.kind == KeywordArgumentKind.NAMED_ONLY:
+            prefix = "🏷"
+        elif self.kind == KeywordArgumentKind.POSITIONAL_ONLY:
+            prefix = "⟶"
         return (
             f"{prefix}{self.name!s}"
             f"{(': ' + (' | '.join(f'{s}' for s in self.types))) if self.types else ''}"
@@ -335,8 +430,14 @@ class ArgumentSpec:
         resolve_named: bool = True,
         resolve_variables_until: Any = None,
         dict_to_kwargs: bool = False,
-    ) -> Any:
-        from robot.running.arguments.argumentresolver import ArgumentResolver
+        validate: bool = True,
+    ) -> Tuple[List[Any], Dict[str, Any]]:
+        from robot.running.arguments.argumentresolver import (
+            ArgumentResolver,
+            DictToKwargs,
+            NamedArgumentResolver,
+            VariableReplacer,
+        )
         from robot.running.arguments.argumentspec import (
             ArgumentSpec as RobotArgumentSpec,
         )
@@ -354,20 +455,30 @@ class ArgumentSpec:
                 self.types,
             )
         self.__robot_arguments.name = self.name
-        resolver = ArgumentResolver(
-            self.__robot_arguments,
-            resolve_named=resolve_named,
-            resolve_variables_until=resolve_variables_until,
-            dict_to_kwargs=dict_to_kwargs,
-        )
-        resolver.resolve(arguments, variables)
+        if validate:
+            resolver = ArgumentResolver(
+                self.__robot_arguments,
+                resolve_named=resolve_named,
+                resolve_variables_until=resolve_variables_until,
+                dict_to_kwargs=dict_to_kwargs,
+            )
+            return cast(Tuple[List[Any], Dict[str, Any]], resolver.resolve(arguments, variables))
+
+        positional, named = NamedArgumentResolver(self.__robot_arguments).resolve(arguments, variables)
+        positional, named = VariableReplacer(resolve_variables_until).replace(positional, named, variables)
+        positional, named = DictToKwargs(self.__robot_arguments, dict_to_kwargs).handle(positional, named)
+        return positional, named
 
 
 @dataclass
 class KeywordDoc(SourceEntity):
     name: str = ""
     name_token: Optional[Token] = field(default=None, compare=False)
-    args: List[KeywordArgumentDoc] = field(default_factory=list, compare=False)
+    arguments: List[ArgumentInfo] = field(default_factory=list, compare=False)
+    arguments_spec: Optional[ArgumentSpec] = field(default=None, compare=False)
+    argument_definitions: Optional[List[ArgumentDefinition]] = field(
+        default=None, compare=False, metadata={"nosave": True}
+    )
     doc: str = field(default="", compare=False)
     tags: List[str] = field(default_factory=list)
     type: str = "keyword"
@@ -383,15 +494,44 @@ class KeywordDoc(SourceEntity):
     is_registered_run_keyword: bool = field(default=False, compare=False)
     args_to_process: Optional[int] = field(default=None, compare=False)
     deprecated: bool = field(default=False, compare=False)
-    arguments: Optional[ArgumentSpec] = field(default=None, compare=False)
-    argument_definitions: Optional[List[ArgumentDefinition]] = field(
-        default=None, compare=False, repr=False, hash=False
-    )
 
-    parent: Optional[str] = field(default=None, metadata={"nosave": True})
+    parent_digest: Optional[str] = field(default=None, init=False, metadata={"nosave": True})
+    parent: Optional[LibraryDoc] = field(default=None, init=False, metadata={"nosave": True})
+
+    def _get_argument_definitions(self) -> Optional[List[ArgumentDefinition]]:
+        return (
+            [
+                LibraryArgumentDefinition(
+                    self.line_no if self.line_no is not None else -1,
+                    col_offset=-1,
+                    end_line_no=-1,
+                    end_col_offset=-1,
+                    source=self.source,
+                    name="${" + a.name + "}",
+                    name_token=None,
+                    keyword_doc=self,
+                )
+                for a in self.arguments
+            ]
+            if self.arguments_spec is not None and not self.is_error_handler and self.arguments
+            else []
+        )
+
+    digest: Optional[str] = field(init=False)
+
+    def __post_init__(self) -> None:
+        s = (
+            f"{self.name}|{self.source}|{self.line_no}|"
+            f"{self.end_line_no}|{self.col_offset}|{self.end_col_offset}|"
+            f"{self.type}|{self.libname}|{self.libtype}"
+        )
+        self.digest = hashlib.sha224(s.encode("utf-8")).hexdigest()
+
+        if self.argument_definitions is None:
+            self.argument_definitions = self._get_argument_definitions()
 
     def __str__(self) -> str:
-        return f"{self.name}({', '.join(str(arg) for arg in self.args)})"
+        return f"{self.name}({', '.join(str(arg) for arg in self.arguments)})"
 
     @property
     def matcher(self) -> KeywordMatcher:
@@ -485,7 +625,7 @@ class KeywordDoc(SourceEntity):
             else:
                 result = ""
 
-        if self.args:
+        if self.arguments:
             result += f"\n##{'#'*header_level} Arguments: \n"
 
             result += "\n| | | | |"
@@ -493,12 +633,19 @@ class KeywordDoc(SourceEntity):
 
             escaped_pipe = " \\| "
 
-            for a in self.args:
+            for a in self.arguments:
                 prefix = ""
+                if a.kind in [KeywordArgumentKind.NAMED_ONLY_MARKER, KeywordArgumentKind.POSITIONAL_ONLY_MARKER]:
+                    continue
+
                 if a.kind == KeywordArgumentKind.VAR_POSITIONAL:
                     prefix = "*"
                 elif a.kind == KeywordArgumentKind.VAR_NAMED:
                     prefix = "**"
+                elif a.kind == KeywordArgumentKind.NAMED_ONLY:
+                    prefix = "🏷"
+                elif a.kind == KeywordArgumentKind.POSITIONAL_ONLY:
+                    prefix = "⟶"
 
                 result += (
                     f"\n| `{prefix}{a.name!s}`"
@@ -519,11 +666,27 @@ class KeywordDoc(SourceEntity):
 
     @property
     def signature(self) -> str:
-        return f"({self.type}) \"{self.name}\": ({', '.join(str(a) for a in self.args)})"
+        return (
+            f'({self.type}) "{self.name}": ('
+            + ", ".join(
+                str(a)
+                for a in self.arguments
+                if a.kind not in [KeywordArgumentKind.POSITIONAL_ONLY_MARKER, KeywordArgumentKind.NAMED_ONLY_MARKER]
+            )
+            + ")"
+        )
 
     @property
     def parameter_signature(self) -> str:
-        return f"({', '.join(str(a) for a in self.args)})"
+        return (
+            "("
+            + ", ".join(
+                str(a)
+                for a in self.arguments
+                if a.kind not in [KeywordArgumentKind.POSITIONAL_ONLY_MARKER, KeywordArgumentKind.NAMED_ONLY_MARKER]
+            )
+            + ")"
+        )
 
     def is_reserved(self) -> bool:
         return self.libname == RESERVED_LIBRARY_NAME
@@ -574,7 +737,9 @@ class KeywordDoc(SourceEntity):
 
 
 class KeywordError(Exception):
-    pass
+    def __init__(self, *args: Any, multiple_keywords: Optional[List[KeywordDoc]] = None, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+        self.multiple_keywords = multiple_keywords
 
 
 @dataclass
@@ -611,7 +776,7 @@ class KeywordStore:
             file_info = "File"
         error = [f"{file_info} contains multiple keywords matching name '{key}':"]
         names = sorted(k.name for k, v in items)
-        raise KeywordError("\n    ".join(error + names))
+        raise KeywordError("\n    ".join(error + names), multiple_keywords=[v for _, v in items])
 
     def __contains__(self, __x: object) -> bool:
         return any(k == __x for k in self._matchers.keys())
@@ -676,16 +841,44 @@ class LibraryDoc:
     source: Optional[str] = None
     line_no: int = -1
     end_line_no: int = -1
-    inits: KeywordStore = field(default_factory=KeywordStore, compare=False)
-    keywords: KeywordStore = field(default_factory=KeywordStore, compare=False)
+    _inits: KeywordStore = field(default_factory=KeywordStore, compare=False)
+    _keywords: KeywordStore = field(default_factory=KeywordStore, compare=False)
+    types: List[TypeDoc] = field(default_factory=list)
     module_spec: Optional[ModuleSpec] = None
     errors: Optional[List[Error]] = field(default=None, compare=False)
     python_path: Optional[List[str]] = None
     stdout: Optional[str] = field(default=None, compare=False)
     has_listener: Optional[bool] = None
     library_type: Optional[LibraryType] = None
+
     digest: Optional[str] = field(init=False)
 
+    @property
+    def inits(self) -> KeywordStore:
+        return self._inits
+
+    @inits.setter
+    def inits(self, value: KeywordStore) -> None:
+        self._inits = value
+        self._update_keywords(self._inits)
+
+    @property
+    def keywords(self) -> KeywordStore:
+        return self._keywords
+
+    @keywords.setter
+    def keywords(self, value: KeywordStore) -> None:
+        self._keywords = value
+        self._update_keywords(self._keywords)
+
+    def _update_keywords(self, keywords: Optional[Iterable[KeywordDoc]]) -> None:
+        if not keywords:
+            return
+
+        for k in keywords:
+            k.parent = self
+            k.parent_digest = self.digest
+
     def __post_init__(self) -> None:
         s = (
             f"{self.name}|{self.source}|{self.line_no}|"
@@ -694,6 +887,9 @@ class LibraryDoc:
         )
         self.digest = hashlib.sha224(s.encode("utf-8")).hexdigest()
 
+        self._update_keywords(self._inits)
+        self._update_keywords(self._keywords)
+
     @single_call
     def __hash__(self) -> int:
         return hash(
@@ -709,6 +905,12 @@ class LibraryDoc:
             )
         )
 
+    def get_types(self, type_names: Optional[List[str]]) -> List[TypeDoc]:
+        if not type_names:
+            return []
+
+        return [t for t in self.types if t.name in type_names]
+
     @property
     def is_deprecated(self) -> bool:
         return DEPRECATED_PATTERN.match(self.doc) is not None
@@ -735,7 +937,7 @@ class LibraryDoc:
             def write_lines(*args: str) -> None:
                 result.writelines(i + "\n" for i in args)
 
-            if add_signature and any(v for v in self.inits.values() if v.args):
+            if add_signature and any(v for v in self.inits.values() if v.arguments):
                 for i in self.inits.values():
                     write_lines(i.to_markdown(header_level=header_level), "", "---")
 
@@ -806,7 +1008,7 @@ class LibraryDoc:
 
     def _get_doc_for_keywords(self, header_level: int = 2) -> str:
         result = ""
-        if any(v for v in self.inits.values() if v.args):
+        if any(v for v in self.inits.values() if v.arguments):
             result += "\n---\n\n"
             result += f"\n##{'#'*header_level} Importing\n\n"
 
@@ -841,7 +1043,7 @@ class LibraryDoc:
         entries = re.findall(r"^##\s+(.+)", doc, flags=re.MULTILINE)
 
         if not only_doc:
-            if any(v for v in self.inits.values() if v.args):
+            if any(v for v in self.inits.values() if v.arguments):
                 entries.append("Importing")
             if self.keywords:
                 entries.append("Keywords")
@@ -1439,11 +1641,13 @@ def get_library_doc(
                 elif isinstance(lib, robot.running.testlibraries._HybridLibrary):
                     libdoc.library_type = LibraryType.HYBRID
 
+                init_wrappers = [KeywordWrapper(lib.init, source)]
+                init_keywords = [(KeywordDocBuilder().build_keyword(k), k) for k in init_wrappers]
                 libdoc.inits = KeywordStore(
                     keywords=[
                         KeywordDoc(
                             name=libdoc.name,
-                            args=[KeywordArgumentDoc.from_robot(a) for a in kw[0].args],
+                            arguments=[ArgumentInfo.from_robot(a) for a in kw[0].args],
                             doc=kw[0].doc,
                             tags=list(kw[0].tags),
                             source=kw[0].source,
@@ -1457,12 +1661,9 @@ def get_library_doc(
                             longname=f"{libdoc.name}.{kw[0].name}",
                             doc_format=str(lib.doc_format) or ROBOT_DOC_FORMAT,
                             is_initializer=True,
-                            arguments=ArgumentSpec.from_robot_argument_spec(kw[1].arguments),
-                            parent=libdoc.digest,
+                            arguments_spec=ArgumentSpec.from_robot_argument_spec(kw[1].arguments),
                         )
-                        for kw in [
-                            (KeywordDocBuilder().build_keyword(k), k) for k in [KeywordWrapper(lib.init, source)]
-                        ]
+                        for kw in init_keywords
                     ]
                 )
 
@@ -1480,13 +1681,15 @@ def get_library_doc(
                             )
                         )
 
+                keyword_wrappers = [KeywordWrapper(k, source) for k in lib.handlers]
+                keyword_docs = [(KeywordDocBuilder().build_keyword(k), k) for k in keyword_wrappers]
                 libdoc.keywords = KeywordStore(
                     source=libdoc.name,
                     source_type=libdoc.type,
                     keywords=[
                         KeywordDoc(
                             name=kw[0].name,
-                            args=[KeywordArgumentDoc.from_robot(a) for a in kw[0].args],
+                            arguments=[ArgumentInfo.from_robot(a) for a in kw[0].args],
                             doc=kw[0].doc,
                             tags=list(kw[0].tags),
                             source=kw[0].source,
@@ -1504,18 +1707,54 @@ def get_library_doc(
                             is_registered_run_keyword=RUN_KW_REGISTER.is_run_keyword(libdoc.name, kw[0].name),
                             args_to_process=RUN_KW_REGISTER.get_args_to_process(libdoc.name, kw[0].name),
                             deprecated=kw[0].deprecated,
-                            arguments=ArgumentSpec.from_robot_argument_spec(kw[1].arguments)
+                            arguments_spec=ArgumentSpec.from_robot_argument_spec(kw[1].arguments)
                             if not kw[1].is_error_handler
                             else None,
-                            parent=libdoc.digest,
                         )
-                        for kw in [
-                            (KeywordDocBuilder().build_keyword(k), k)
-                            for k in [KeywordWrapper(k, source) for k in lib.handlers]
-                        ]
+                        for kw in keyword_docs
                     ],
                 )
 
+                if get_robot_version() >= (5, 0):
+                    from robot.libdocpkg.datatypes import TypeDoc as RobotTypeDoc
+                    from robot.running.arguments.argumentspec import TypeInfo
+
+                    def _yield_type_info(info: TypeInfo) -> TypeInfo:
+                        if not info.is_union:
+                            yield info
+                        for nested in info.nested:
+                            yield from _yield_type_info(nested)
+
+                    def _get_type_docs(keywords: List[Any], custom_converters: List[Any]) -> Set[RobotTypeDoc]:
+                        type_docs: Dict[RobotTypeDoc, Set[str]] = {}
+                        for kw in keywords:
+                            for arg in kw.args:
+                                kw.type_docs[arg.name] = {}
+                                for type_info in _yield_type_info(arg.type):
+                                    type_doc = RobotTypeDoc.for_type(type_info.type, custom_converters)
+                                    if type_doc:
+                                        kw.type_docs[arg.name][type_info.name] = type_doc.name
+                                        type_docs.setdefault(type_doc, set()).add(kw.name)
+                        for type_doc, usages in type_docs.items():
+                            type_doc.usages = sorted(usages, key=str.lower)
+                        return set(type_docs)
+
+                    libdoc.types = [
+                        TypeDoc(
+                            td.type,
+                            td.name,
+                            td.doc,
+                            td.accepts,
+                            td.usages,
+                            [EnumMember(m.name, m.value) for m in td.members] if td.members else None,
+                            [TypedDictItem(i.key, i.type, i.required) for i in td.items] if td.items else None,
+                            libname=libdoc.name,
+                            libtype=libdoc.type,
+                            doc_format=libdoc.doc_format,
+                        )
+                        for td in _get_type_docs([kw[0] for kw in keyword_docs + init_keywords], lib.converters)
+                    ]
+
             except (SystemExit, KeyboardInterrupt):
                 raise
             except BaseException as e:
@@ -1713,7 +1952,7 @@ def get_variables_doc(
                         keywords=[
                             KeywordDoc(
                                 name=libdoc.name,
-                                args=[KeywordArgumentDoc.from_robot(a) for a in kw[0].args],
+                                arguments=[ArgumentInfo.from_robot(a) for a in kw[0].args],
                                 doc=kw[0].doc,
                                 source=kw[0].source,
                                 line_no=kw[0].lineno if kw[0].lineno is not None else -1,
@@ -1725,8 +1964,7 @@ def get_variables_doc(
                                 libtype=libdoc.type,
                                 longname=f"{libdoc.name}.{kw[0].name}",
                                 is_initializer=True,
-                                arguments=ArgumentSpec.from_robot_argument_spec(kw[1].arguments),
-                                parent=libdoc.digest,
+                                arguments_spec=ArgumentSpec.from_robot_argument_spec(kw[1].arguments),
                             )
                             for kw in [
                                 (KeywordDocBuilder().build_keyword(k), k)
@@ -1751,7 +1989,7 @@ def get_variables_doc(
                             keywords=[
                                 KeywordDoc(
                                     name=libdoc.name,
-                                    args=[],
+                                    arguments=[],
                                     doc=kw[0].doc,
                                     source=kw[0].source,
                                     line_no=kw[0].lineno if kw[0].lineno is not None else -1,
@@ -1763,7 +2001,6 @@ def get_variables_doc(
                                     libtype=libdoc.type,
                                     longname=f"{libdoc.name}.{kw[0].name}",
                                     is_initializer=True,
-                                    parent=libdoc.digest,
                                 )
                                 for kw in [
                                     (KeywordDocBuilder().build_keyword(k), k)
@@ -2098,3 +2335,165 @@ def complete_variables_import(
             ]
 
     return list(set(result))
+
+
+def get_model_doc(
+    model: ast.AST,
+    source: str,
+    model_type: str = "RESOURCE",
+    scope: str = "GLOBAL",
+    append_model_errors: bool = True,
+) -> LibraryDoc:
+    from robot.errors import DataError, VariableError
+    from robot.libdocpkg.robotbuilder import KeywordDocBuilder
+    from robot.parsing.lexer.tokens import Token as RobotToken
+    from robot.parsing.model.blocks import Keyword
+    from robot.parsing.model.statements import Arguments, KeywordName
+    from robot.running.builder.transformers import ResourceBuilder
+    from robot.running.model import ResourceFile
+    from robot.running.usererrorhandler import UserErrorHandler
+    from robot.running.userkeyword import UserLibrary
+
+    errors: List[Error] = []
+    keyword_name_nodes: List[KeywordName] = []
+    keywords_nodes: List[Keyword] = []
+    for node in ast.walk(model):
+        if isinstance(node, Keyword):
+            keywords_nodes.append(node)
+        if isinstance(node, KeywordName):
+            keyword_name_nodes.append(node)
+
+        error = node.error if isinstance(node, HasError) else None
+        if error is not None:
+            errors.append(Error(message=error, type_name="ModelError", source=source, line_no=node.lineno))
+        if append_model_errors:
+            node_errors = node.errors if isinstance(node, HasErrors) else None
+            if node_errors is not None:
+                for e in node_errors:
+                    errors.append(Error(message=e, type_name="ModelError", source=source, line_no=node.lineno))
+
+    def get_keyword_name_token_from_line(line: int) -> Optional[Token]:
+        for keyword_name in keyword_name_nodes:
+            if keyword_name.lineno == line:
+                return cast(Token, keyword_name.get_token(RobotToken.KEYWORD_NAME))
+
+        return None
+
+    def get_argument_definitions_from_line(line: int) -> List[ArgumentDefinition]:
+        keyword_node = next((k for k in keywords_nodes if k.lineno == line), None)
+        if keyword_node is None:
+            return []
+
+        arguments_node = next((n for n in ast.walk(keyword_node) if isinstance(n, Arguments)), None)
+        if arguments_node is None:
+            return []
+
+        args: List[str] = []
+        arguments = arguments_node.get_tokens(RobotToken.ARGUMENT)
+        argument_definitions = []
+
+        for argument_token in (cast(RobotToken, e) for e in arguments):
+            try:
+                argument = get_variable_token(argument_token)
+
+                if argument is not None and argument.value != "@{}":
+                    if argument.value not in args:
+                        args.append(argument.value)
+                        arg_def = ArgumentDefinition(
+                            name=argument.value,
+                            name_token=strip_variable_token(argument),
+                            line_no=argument.lineno,
+                            col_offset=argument.col_offset,
+                            end_line_no=argument.lineno,
+                            end_col_offset=argument.end_col_offset,
+                            source=source,
+                        )
+                        argument_definitions.append(arg_def)
+
+            except VariableError:
+                pass
+
+        return argument_definitions
+
+    res = ResourceFile(source=source)
+
+    ResourceBuilder(res).visit(model)
+
+    class MyUserLibrary(UserLibrary):
+        current_kw: Any = None
+
+        def _log_creating_failed(self, handler: UserErrorHandler, error: BaseException) -> None:
+            err = Error(
+                message=f"Creating keyword '{handler.name}' failed: {error!s}",
+                type_name=type(error).__qualname__,
+                source=self.current_kw.source if self.current_kw is not None else None,
+                line_no=self.current_kw.lineno if self.current_kw is not None else None,
+            )
+            errors.append(err)
+
+        def _create_handler(self, kw: Any) -> Any:
+            self.current_kw = kw
+            try:
+                handler = super()._create_handler(kw)
+                setattr(handler, "errors", None)
+            except DataError as e:
+                err = Error(
+                    message=str(e),
+                    type_name=type(e).__qualname__,
+                    source=kw.source,
+                    line_no=kw.lineno,
+                )
+                errors.append(err)
+
+                handler = UserErrorHandler(e, kw.name, self.name)
+                handler.source = kw.source
+                handler.lineno = kw.lineno
+
+                setattr(handler, "errors", [err])
+
+            return handler
+
+    lib = MyUserLibrary(res)
+
+    libdoc = LibraryDoc(
+        name=lib.name or "",
+        doc=lib.doc,
+        type=model_type,
+        scope=scope,
+        source=source,
+        line_no=1,
+        errors=errors,
+    )
+
+    libdoc.keywords = KeywordStore(
+        source=libdoc.name,
+        source_type=libdoc.type,
+        keywords=[
+            KeywordDoc(
+                name=kw[0].name,
+                arguments=[ArgumentInfo.from_robot(a) for a in kw[0].args],
+                doc=kw[0].doc,
+                tags=list(kw[0].tags),
+                source=str(kw[0].source),
+                name_token=get_keyword_name_token_from_line(kw[0].lineno),
+                line_no=kw[0].lineno if kw[0].lineno is not None else -1,
+                col_offset=-1,
+                end_col_offset=-1,
+                end_line_no=-1,
+                libname=libdoc.name,
+                libtype=libdoc.type,
+                longname=f"{libdoc.name}.{kw[0].name}",
+                is_embedded=is_embedded_keyword(kw[0].name),
+                errors=getattr(kw[1], "errors") if hasattr(kw[1], "errors") else None,
+                is_error_handler=isinstance(kw[1], UserErrorHandler),
+                error_handler_message=str(cast(UserErrorHandler, kw[1]).error)
+                if isinstance(kw[1], UserErrorHandler)
+                else None,
+                arguments_spec=ArgumentSpec.from_robot_argument_spec(kw[1].arguments),
+                argument_definitions=get_argument_definitions_from_line(kw[0].lineno),
+            )
+            for kw in [(KeywordDocBuilder(resource=True).build_keyword(lw), lw) for lw in lib.handlers]
+        ],
+    )
+
+    return libdoc
